
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   compute_homography
%   by Petra Kocmanova
%   last modified: 27.6.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [H] = compute_homography(pp, xx)
%Compute matrix of homography H [3 x 3] from coresponding points on image
%[pp] and calibration plate [xx]

pp = pp(1:2,:) ./ repmat(pp(3,:),2,1);
xx = xx(1:2,:);

corner_number = size(pp,2);
assert(corner_number==size(xx,2));

%Normalizations
pp_mean = mean(pp,2);
pdist = mean( sum((pp - repmat(pp_mean,1,corner_number)).^2,1).^0.5 );
pscale = 2^0.5/pdist;
Tp = [eye(2)*pscale, -pp_mean*pscale; 0 0 1];
pp_norm = Tp*[pp; ones(1,corner_number)];

xx_mean = mean(xx,2);
xdist = mean( sum((xx - repmat(xx_mean,1,corner_number)).^2,1).^0.5 );
xscale = 2^0.5/xdist;
Tx = [eye(2)*xscale, -xx_mean*xscale; 0 0 1];
xx_norm = Tx*[xx; ones(1,corner_number)];


A = zeros(2*corner_number,9);
for i=1:corner_number
    A(i*2-1, 4:6) = -pp_norm(3,i)*xx_norm(:,i)';
    A(i*2-1, 7:9) = pp_norm(2,i)*xx_norm(:,i)';
    
    A(i*2, 1:3) = pp_norm(3,i)*xx_norm(:,i)';
    A(i*2, 7:9) = -pp_norm(1,i)*xx_norm(:,i)';
end

[~,~,v] = svd(A);
h = v(:,end);
Hn = reshape(h,[3,3])';
H = Tp\Hn*Tx;
H = H/norm(H(:,1));

end

