
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   initial_calibration
%   by Petra Kocmanova
%   last modified: 27.6.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function  initial_calibration(V,Hi,k)
%Compute calibration parameters from one image
%   Compute camera matrix K[3x3], rotation matrix R[3x3] and translation
%   t[3x1] matrix between coordinate frame of one image and coordinate
%   frame of calibration plate


global cameras
global images


[~,~,vv] = svd(V,0);
b = vv(:,end); %b = [B11,B12,B22,B13,B23,B33]T

v0 = (b(2)*b(4)-b(1)*b(5)) / (b(1)*b(3)-b(2)^2);
lambda = b(6) - (b(4)^2+v0*(b(2)*b(4)-b(1)*b(5)))/b(1);
alpha = (lambda/b(1))^0.5;
beta = (lambda*b(1)/(b(1)*b(3)-b(2)^2))^0.5;
c = -b(2)*alpha^2*beta/lambda;
u0 = c*v0/alpha-b(4)*alpha^2/lambda;

cameras(k,1).K0 = [alpha c u0; 0 beta v0; 0 0 1];



R0 = zeros(3,3);
hom_im_number=size(Hi,2);

for i=1:hom_im_number
    
    R0(:,1) = cameras(k,1).K0\Hi{k,i}(:,1);
    R0(:,2) = cameras(k,1).K0\Hi{k,i}(:,2);
    lambda = 1/norm(R0(:,1));
    R0(:,1:2) = lambda*R0(:,1:2);
    R0(:,3) = cross(R0(:,1),R0(:,2));
    [U,~,V] = svd(R0);
    images(i,k).R0 = U*V';
    images(i,k).t0 = lambda*(cameras(k,1).K0\Hi{k,i}(:,3));
    
end


end

