
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   refine_calibration
%   by Petra Kocmanova
%   last modified: 27.6.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function refine_calibration(k0,k)
%Refine camera matrix cameras().K, distortion coefficients
% cameras().k, rotation matrix images().R and translation matrix images().t 
% between calibration plate object coordinate system and cameras image
% coordinate system

global cameras
global images
global use_im

cam_par0 = [];
cam_par0.K = cameras(k,1).K0;
cam_par0.kc = k0;

images_number=size(images,1);


j=1;
for i=1:images_number
    if use_im(k,i)
        [images(i,k).H]=compute_homography(images(i,k).grid_p,images(i,k).grid_x);
        cam_par0.R{1,j}=images(i,k).R0;
        cam_par0.t{1,j}=images(i,k).t0;
        j=j+1;
    end
end

init_data=[];

%Camera matrix
init_data(1)=cam_par0.K(1,1);
init_data(2)=cam_par0.K(2,2);
init_data(3)=cam_par0.K(1,3);
init_data(4)=cam_par0.K(2,3);

%distortion coefficeints
coef_number = min([4 length(cam_par0.kc)]);%Do not estimate 5th coefficient
init_data(5:(5+coef_number-1))=cam_par0.kc(1:coef_number);

%extrinsics
j=size(cam_par0.R,2);
l=5+coef_number;

for i=1:j
    [V,D]=eig(cam_par0.R{1,i}-eye(3));
    evs=diag(D);
    [~,mini]=min(evs);
    vect=V(:,mini);
    
    cosphi=0.5*(trace(cam_par0.R{1,i})-1);
    sinphi=0.5*vect'*[ cam_par0.R{1,i}(3,2)-cam_par0.R{1,i}(2,3);...
        cam_par0.R{1,i}(1,3)-cam_par0.R{1,i}(3,1); cam_par0.R{1,i}(2,1)-cam_par0.R{1,i}(1,2)];
    phi=atan2(sinphi,cosphi);
    r=phi*vect;
    
    init_data(l:(l+2))=r;
    init_data((l+3):(l+5))=cam_par0.t{1,i};
    l=l+6;
end


%Minimize
minimization_options=optimset('LargeScale','off',...
    'Algorithm','levenberg-marquardt',...
    'Display','iter', ...
    'TolFun',1e-4,...
    'TolX',1e-8,...
    'MaxFunEvals',20000,...
    'MaxIter',100);


[refine_data,~,rep_error] = lsqnonlin(@(x) cost(x,cam_par0,k),init_data,[],[],minimization_options);

%Camera matrix K
cameras(k,1).K=[refine_data(1) 0 refine_data(3);
    0 refine_data(2) refine_data(4);
    0 0 1];

%Distortion coefficient
coef_number = min([4 length(cam_par0.kc)]);
cameras(k,1).k=[refine_data(5:(5+coef_number-1)) zeros(1,5-coef_number)];

%Extrinsic parameters
l=5+coef_number;


[images_number]=size(images,1);
for i=1:images_number
    if use_im(k,i)
        R=rotationmat(refine_data(l:l+2));
        images(i,k).R=R;
        images(i,k).t=refine_data(l+3:l+5)';
        l=l+6;
    end
end
cameras(k,1).rep_error=rep_error;

end

