
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   refine_extrinsic_calibration
%   by Petra Kocmanova
%   last modified: 27.6.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [blok] = refine_extrinsic_calibration(param0)
%Refine camera matrix cameras().Kref, rotation matrix cameras().Rref and 
% translation matrix cameras().tref between reference camera coordinate
% system and coordinate system of all other calibrated cameras

global images
global cameras
global use_im
global path_calibration_data

[images_number, cameras_number]=size(images); 
init_data_ext=[];
vect=1;


%Intrinsic parameters
for k=1:cameras_number
    init_data_ext(vect:vect+3)=[param0.K{k}(1,1) param0.K{k}(2,2)...
        param0.K{k}(1,3) param0.K{k}(2,3)];
    vect=vect+4;
end

for i=1:images_number
    if use_im(1,i)
        init_data_ext(vect:vect+2)=rotationpars(param0.Rext{i});
        vect=vect+3;
        init_data_ext(vect:vect+2)=param0.text{i};
        vect=vect+3;
    end
end

for k=2:cameras_number
    init_data_ext(vect:vect+2)=(rotationpars(param0.R{k}))';
    vect=vect+3;
    init_data_ext(vect:vect+2)=param0.t{k};
    vect=vect+3;
end

  %Minimize
  minimization_options=optimset('LargeScale','off',...
    'Algorithm','levenberg-marquardt',...
    'Display','iter', ...
    'TolFun',1e-4,...
    'TolX',1e-8,...
    'MaxFunEvals',20000,...
    'MaxIter',100);

refine_data_ext=lsqnonlin(@(x) cost_ext(x,param0),init_data_ext,[],[],minimization_options);


%Decode refine_data_ext
vect=1;
for k=1:cameras_number
    cameras(k,1).Kref=[refine_data_ext(vect) 0 refine_data_ext(vect+2);
        0  refine_data_ext(vect+1) refine_data_ext(vect+3);
        0 0 1];
    vect=vect+4;   
end

%Translations and rotations between reference frame of calibration plate
%and reference camera
for i=1:images_number
    if use_im(1,i)
        images(i,1).R=rotationmat(refine_data_ext(vect:vect+2));
        vect=vect+3;
        images(i,1).t=refine_data_ext(vect:vect+2)';
        vect=vect+3;   
    end
end

for k=2:cameras_number
    cameras(k,1).Rref=rotationmat(refine_data_ext(vect:vect+2));
    vect=vect+3;
    cameras(k,1).tref=refine_data_ext(vect:vect+2)';
    vect=vect+3;
end

%Computation of reprojection error
for k=1:cameras_number
    cameras(k,1).rep_error_ext=[];  
    kk=cameras(k,1).k;
    K=cameras(k,1).Kref;
    for i=1:images_number
        if use_im(1,i)
            X=images(i,k).grid_x;
            R=cameras(k,1).Rref'*images(i,1).R;
            t=cameras(k,1).Rref'*(images(i,1).t - cameras(k,1).tref);
            
            pp = project_points(X,K,kk,R,t);
            rep_error=images(i,k).grid_p(1:2,:)-pp;   
            cameras(k,1).rep_error_ext=[cameras(k,1).rep_error_ext rep_error];  
        end
    end
    rep_error=[cameras(k,1).rep_error_ext(1,:)';cameras(k,1).rep_error_ext(2,:)'];
    cameras(k,1).sd_rep_error_ext=sqrt((rep_error'*rep_error)/size(rep_error,1));
end

%Export results to excel
blok=cell(17,cameras_number+1);
blok(1:17,1)={'name of camera','fx [px]','fy [px]','u0 [px]','v0 [px]',...
    'k1','k2','p1','p2','rX [g]','rY [g]','rZ [g]','tX [m]', 'tY [m]','tZ [m]',...
    'SD1 [px]','SD2 [px]'};

for k=1:cameras_number
    blok(1,k+1)=cellstr(cameras(k,1).name(1:end-4));
    blok(2:5,k+1)=num2cell(round(100*[cameras(k,1).Kref(1,1);cameras(k,1).Kref(2,2);...
        cameras(k,1).Kref(1,3);cameras(k,1).Kref(2,3)])/100);
    blok(6:9,k+1)=num2cell(round(10000*cameras(k,1).k(1,1:4))/10000)';
    blok(10:12,k+1)=num2cell(round(10000*rotationpars(cameras(k,1).Rref)*200/pi())/10000);
    blok(13:15,k+1)=num2cell(round(1000*cameras(k,1).tref)/1000);
    blok(16:17,k+1)=num2cell(round(100*[std(cameras(k,1).rep_error,1);cameras(k,1).sd_rep_error_ext])/100);
    
end

xlswrite([path_calibration_data '\Mutual_calibration_RESULTS'],blok);
xlswrite('Mutual_calibration_RESULTS',blok);

save

end

